unit fList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, ComObj;

type
  TfrmList = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    ListView1: TListView;
    Label1: TLabel;
    chOwnerData: TCheckBox;
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure chOwnerDataClick(Sender: TObject);
  private
    { Private declarations }
    FDoc: Variant;
    FTable: Variant;
    FNodeStyle: Boolean;
    procedure FillRecords;
    procedure FillNodes;
    procedure FillItemFromNode(ANode: Variant; AListItem: TListItem);
    procedure FillItemFromRecord(ATreeItem: Variant; AListItem: TListItem;
      ATableDef: Variant);
  public
    { Public declarations }
    class procedure Execute;
  end;

var
  frmList: TfrmList;

implementation

{$R *.DFM}

{ TfrmList }

class procedure TfrmList.Execute;
var
  WOrg: Variant;
  bMustClose: Boolean;
begin
  WOrg := CreateOLEObject('WinOrganizer.App');
  Application.Handle := WOrg.Handle;

  bMustClose := False;

  with TfrmList.Create(Application) do
  try
    if WOrg.FileList.ActiveIndex < 0 then
      Label1.Caption :=  'No file selected'
    else
    begin
      FDoc := WOrg.FileList.ActiveFile.Selected;
      if VarIsEmpty(FDoc) then
        Label1.Caption :=  'No document selected'
      else
      begin
        ListView1.Visible := True;
        if FDoc.isTable and WOrg.FileList.ActiveFile.TableExists(FDoc.Ext) then
        begin
          FTable := FDoc.Open;
          FillRecords;
        end
        else
          FillNodes;
      end;
    end;

    ShowModal;
  finally
    if not (VarIsEmpty(FTable) or VarIsNull(FTable)) then
    begin
      FTable := NULL;
      FDoc.Close;
    end;
    Free;
  end;
end;

procedure TfrmList.FillItemFromNode(ANode: Variant; AListItem: TListItem);
begin
  AListItem.Caption := ANode.Text;
  AListItem.SubItems.Add(ANode.Ext);
  AListItem.SubItems.Add(ANode.CreateDate);
  AListItem.SubItems.Add(ANode.UpdateDate);
end;

procedure TfrmList.FillNodes;
var
  i: Integer;
begin
  FNodeStyle := True;

  ListView1.Items.BeginUpdate;
  try
    ListView1.Items.Clear;
    ListView1.OwnerData := chOwnerData.Checked;
    if chOwnerData.Checked then
      ListView1.Items.Count := FDoc.NodeCount
    else
    begin
      for i := 0 to FDoc.NodeCount - 1 do
        FillItemFromNode(FDoc.Nodes[i], ListView1.Items.Add);
    end;
  finally
    ListView1.Items.EndUpdate;
  end;

  if ListView1.Columns.Count <> 4 then
  begin
    ListView1.Columns.BeginUpdate;
    try
      with ListView1.Columns.Add do
      begin
        Caption := 'Text';
        Width := 100;
      end;
      with ListView1.Columns.Add do
      begin
        Caption := 'Ext';
        Width := 80;
      end;
      with ListView1.Columns.Add do
      begin
        Caption := 'Created';
        Width := 150;
      end;
      with ListView1.Columns.Add do
      begin
        Caption := 'Updated';
        Width := 150;
      end;
    finally
      ListView1.Columns.EndUpdate;
    end;
  end;
end;

procedure TfrmList.FillItemFromRecord(ATreeItem: Variant;
  AListItem: TListItem; ATableDef: Variant);
var
  i: Integer;
  V, VRecord: Variant;
  S: String;
begin
  VRecord := ATreeItem.Open;
  try
    for i := 0 to ATableDef.FieldCount - 1 do
    begin
      V := ATableDef.Fields[i];
      if V.isBlob then
      begin
        S := 'Blob';
        if not VRecord.isNull(i) then
          S := UpperCase(S);
      end
      else
      if VRecord.isNULL(i) then
        S := ''
      else
        S := VRecord.Values[i];

      if i = 0 then
        AListItem.Caption := S
      else
        AListItem.SubItems.Add(S);
    end;
  finally
    ATreeItem.Close;
  end;
end;

procedure TfrmList.FillRecords;
var
  i: Integer;
  V: Variant;
begin
  FNodeStyle := False;

  ListView1.Items.BeginUpdate;
  try
    ListView1.Items.Clear;
    ListView1.OwnerData := chOwnerData.Checked;
    if chOwnerData.Checked then
      ListView1.Items.Count := FTable.Count
    else
    begin
      for i := 0 to FTable.Count - 1 do
        FillItemFromRecord(FTable.Items[i], ListView1.Items.Add, FTable.TableDef);
    end;
  finally
    ListView1.Items.EndUpdate;
  end;

  V := FTable.TableDef;
  if ListView1.Columns.Count <> V.FieldCount then
  begin
    ListView1.Columns.BeginUpdate;
    try
      for i := 0 to V.FieldCount - 1 do
        with ListView1.Columns.Add do
        begin
          Caption := V.Fields[i].FieldName;
          if V.Fields[i].isBlob then
            Width := ListView1.Canvas.TextWidth('BLOB ')
          else
            Width := ListView1.Canvas.TextWidth('x') * V.Fields[i].DataSize;

          if Width < 50 then
            Width := 50
          else
          if Width > 300 then
            Width := 300;
        end;
    finally
      ListView1.Columns.EndUpdate;
    end;
  end;
end;

procedure TfrmList.ListView1Data(Sender: TObject; Item: TListItem);
begin
  if FNodeStyle then
    FillItemFromNode(FDoc.Nodes[Item.Index], Item)
  else
    FillItemFromRecord(FTable.Items[Item.Index], Item, FTable.TableDef);
end;

procedure TfrmList.chOwnerDataClick(Sender: TObject);
begin
  if FNodeStyle then
    FillNodes
  else
    FillRecords;
end;

end.
